import time
import board
import digitalio
import gc
import busio

from adafruit_hid.keyboard import Keyboard
from adafruit_hid.keycode import Keycode
from adafruit_hid.mouse import Mouse
from adafruit_neotrellis.neotrellis import NeoTrellis

OFF = (1, 0, 0)
LIT = (40, 20, 0)
SWITCH_PINS = (board.D5, board.D6, board.D9, board.D10, board.D11, board.D12, board.A1)
BOUNCE_SECS = 0.100

class SnakeSwitch:
    def __init__(self):
        i2c_bus = busio.I2C(board.SCL, board.SDA)
        self.trellis = NeoTrellis(i2c_bus)

        self.switch_ins = tuple(digitalio.DigitalInOut(pin) for pin in SWITCH_PINS)
        for switch_in in self.switch_ins:
            switch_in.switch_to_input(pull=digitalio.Pull.UP)

        self.kbd = Keyboard()
        self.states = {}

        for i in range(16):
            self.states[i] = False
            # activate rising / falling edge events on all keys, set up callback
            self.trellis.activate_key(i, NeoTrellis.EDGE_RISING)
            self.trellis.activate_key(i, NeoTrellis.EDGE_FALLING)
            self.trellis.callbacks[i] = self.toggle

    def toggle(self, event):
        # turn the LED on when a rising edge is detected
        if event.edge == NeoTrellis.EDGE_RISING:
            self.states[event.number] = not self.states[event.number]

            for state, value in self.states.items():
                if value:
                    self.trellis.pixels[state] = LIT
                else:
                    self.trellis.pixels[state] = OFF

    def advance_frame(self):
        for switch, switch_in in enumerate(self.switch_ins):
            self.trellis.sync()
            if not switch_in.value:
                # If switch is pressed, it's pulled low. Debounce by waiting for bounce time.
                time.sleep(BOUNCE_SECS)
                print('switch: ', switch)
                print(gc.mem_free())
                self.kbd.send(Keycode.LEFT_GUI, Keycode.TAB)

                # Wait for switch to be released.
                while not switch_in.value:
                    pass
